/* 
 * "ESCREEN.H"
 * 
 * Part of The Escotia Playstation Library 
 *
 * Functions to make programming a chore not a pleasure
 *
 * Author:	ScoTT Campbell
 * Date:	9/3/98
 *
 * Updated:	28/8/98
 *			11/9/98
 *
*/

#ifndef __ESCSCREENHEADER
#define __ESCSCREENHEADER


// All possible combinations for screen
// Choose video mode then one height, one width and corresponding interlace mode


/****** ONE VIDEO MODE ONLY ******/

#define VIDEO_MODE	(MODE_PAL)
//#define VIDEO_MODE	(MODE_NTSC)



/***** ONE SCREEN HEIGHT ONLY ****/

//#define SCREEN_HEIGHT (240)	// NTSC
//#define SCREEN_HEIGHT (480)	// NTSC
//#define SCREEN_HEIGHT (256)	// PAL
#define SCREEN_HEIGHT (512)	// PAL



/***** ONE SCREEN WIDTH ONLY *****/

//#define SCREEN_WIDTH  (256)
//#define SCREEN_WIDTH  (320)		
//#define SCREEN_WIDTH  (368)	// Manuals say 384 but this is the value when you use TimTool
//#define SCREEN_WIDTH  (512)
#define SCREEN_WIDTH  (640)



/********* INTERLACE MODE ********/

#define	INTERLACED			// Interlace mode - Must be chosen if SCREEN_HEIGHT is 480, or 512	
								// Comment out for non-interlace mode	


/************************* Function Definitions *****************************/

void InitGraphics();
/* Pre:  VIDEO_MODE, SCREEN_HEIGHT, SCREEN_WIDTH, and INTERLACE_MODE above	*/
/*		 are set to a valid combination										*/
/* Post: Yaroze is set up with relevent double buffers, screen mode and the */
/*		 graphics system set up to 16-bit with GPU offset, no dithering,	*/


void Init3dGraphics();
/* Pre:  VIDEO_MODE, SCREEN_HEIGHT, SCREEN_WIDTH, and INTERLACE_MODE above	*/
/*		 are set to a valid combination										*/
/* Post: Yaroze is set up with relevent double buffers, screen mode and the */
/*		 graphics system set up to 16-bit with GPU offset, no dithering		*/


void SetClearScreen(int r, int g, int b);
/* Pre:	 r, g and b are red/green/blue colour values between 0 and 256		*/
/* Post: Screen is cleared to colour (r,g,b) before drawing of OT begins	*/


/********************** End of Function Definitions *************************/




#ifdef	INTERLACED
#define INTERLACE_MODE		(1) 
#endif

#ifndef	INTERLACED
#define INTERLACE_MODE		(0) 
#endif


#endif
